# -*- coding: utf-8 -
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#
# Copyright 2011 Cloudant, Inc.

import logging
import re
import socket
import sys
import threading
import time


log = logging.getLogger(__name__)


class StatsDError(Exception):
    def __init__(self, mesg):
        self.mesg = mesg
    def __str__(self):
        return self.mesg


class BindError(StatsDError):
    pass


class StatsDHandler(threading.Thread):
    def __init__(self, queue, flush_time=10):
        super(StatsDHandler, self).__init__()
        self.setDaemon(True)
        self.queue = queue
        self.lock = threading.Lock()
        self.counters = {}
        self.timers = {}
        self.flush_time = flush_time
        self.key_res = (
            (re.compile("\s+"), "_"),
            (re.compile("\/"), "-"),
            (re.compile("[^a-zA-Z_\-0-9\.]"), "")
        )

    def run(self):
        while True:
            time.sleep(self.flush_time)
            stime = int(time.time())
            with self.lock:
                num_stats = self.enqueue_timers(stime)
                num_stats += self.enqueue_counters(stime)
                self.queue.put(("stats.numStats", num_stats, stime))

    def enqueue_timers(self, stime):
        ret = 0
        for k, v in self.timers.iteritems():
            v.sort()
            pct_thresh = 0.9
            count = len(v)
            vmin, vmax = v[0], v[-1]
            mean, vthres = vmin, vmax

            if count > 1:
                thresh_idx = int((100.0 - pct_thresh) / 100.0 * float(count))
                v = v[:thresh_idx]
                vthresh = v[-1]
                vsum = sum(v)
                mean = vsum / float(len(v))

            self.queue.put(("stats.timers.%s.mean" % k, mean, stime))
            self.queue.put(("stats.timers.%s.upper" % k, vmax, stime))
            t = int(pct_thresh * 100)
            self.queue.put(("stats.timers.%s.upper_%s" % (k,t), vthresh, stime))
            self.queue.put(("stats.timers.%s.lower" % k, vmin, stime))
            self.queue.put(("stats.timers.%s.count" % k, count, stime))
            self.timers[k] = []
            ret += 1

        return ret

    def enqueue_counters(self, stime):
        ret = 0
        for k, v in self.counters.iteritems():
            stat = "stats.%s" % k
            v = v / self.flush_interval
            self.queue.put((stat, v / self.flush_interval, stime))
            self.counters[k] = 0
            ret += 1
        return ret

    def handle(self, data):
        # Adding a bit of extra sauce so clients can
        # send multiple samples in a single UDP
        # packet.
        for line in data.splitlines():
            self.line = line
            if not line.strip():
                continue
            self.handle_line(line)

    def handle_line(self, line):
        bits = line.split(":", 1)
        key = self.handle_key(bits.pop(0))

        if len(bits) == 0:
            self.bad_line()
            return

        # I'm not sure if statsd is doing this on purpose
        # but the code allows for name:v1|t1:v2|t2 etc etc.
        # In the interest of compatibility, I'll maintain
        # the behavior.
        for sample in bits:
            fields = sample.split("|")
            if len(fields) < 2:
                self.bad_line()
                continue
            if fields[1] == "ms":
                self.handle_timer(key, fields)
            else:
                self.handle_counter(key, fields)

    def handle_key(self, key):
        for (rexp, repl) in self.key_res:
            key = rexp.sub(repl, key)
        return key

    def handle_timer(self, key, fields):
        try:
            val = float(fields[0] or 0)
            with self.lock:
                self.timers.setdefault(key, []).append(val)
        except:
            self.bad_line()

    def handle_counter(self, key, fields):
        rate = 1.0
        if len(fields) > 2 and fields[2][:1] == "@":
            try:
                rate = float(fields[2][1:].strip())
            except:
                rate = 1.0
        try:
            val = int(float(fields[0] or 0) / rate)
        except:
            self.bad_line()
            return
        with self.lock:
            if key not in self.counters:
                self.counters[key] = 0
            self.counters[key] += val

    def bad_line(self):
        log.error("StatsD: Invalid line: '%s'" % self.line.strip())


class StatsDServer(threading.Thread):
    def __init__(self, queue, cfg):
        super(StatsDServer, self).__init__()
        self.setDaemon(True)
        self.handler = StatsDHandler(queue, flush_time=cfg["statsd_flush_time"])
        self.handler.start()
        self.sock = self.init_socket(cfg["statsd_ip"], cfg["statsd_port"])

    def init_socket(self, ip, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            sock.bind((ip, port))
            log.info("Opened statsd socket %s:%s" % (ip, port))
            return sock
        except Exception:
            log.error("Error opening statsd socket %s:%s." % (ip, port))
            sys.exit(1)

    def run(self):
        while True:
            data, addr = self.sock.recvfrom(65535)
            self.handler.handle(data)
            if not self.handler.is_alive():
                return



# -*- coding: utf-8 -
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#
# Copyright 2011 Cloudant, Inc.

import logging
import socket
import sys


log = logging.getLogger(__name__)


class CarbonException(Exception):
    def __init__(self, mesg):
        self.mesg = mesg
    def __str__(self):
        return self.mesg


class ConnectError(CarbonException):
    pass


class CarbonClient(object):
    def __init__(self, cfg):
        ip = cfg["graphite_ip"]
        port = cfg["graphite_port"]
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.sock.connect((ip, port))
            log.info("Connect to Carbon at %s:%s" % (ip, port))
        except Exception:
            log.error("Failed to connect to %s:%s" % (ip, port))
            sys.exit(2)

    def send(self, stat, value, mtime):
        mesg = "%s %s %s\n" % (stat, value, mtime)
        self.sock.sendall(mesg)




# -*- coding: utf-8 -
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#
# Copyright 2011 Cloudant, Inc.

import os
import sys

from setuptools import setup

from bucky import __version__

setup(
    name = 'bucky',
    version = __version__,

    description = 'CollectD and StatsD adapter for Graphite',
    long_description = file(
        os.path.join(
            os.path.dirname(__file__),
            'README.rst'
        )
    ).read(),
    author = 'Paul J. Davis',
    author_email = 'paul@cloudant.com',
    license = 'ASF2.0',
    url = 'http://github.com/cloudant/bucky.git',

    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: Internet :: Log Analysis',
        'Topic :: Utilities',
        'Topic :: System :: Networking :: Monitoring'
    ],
    zip_safe = False,
    packages = ['bucky'],
    include_package_data = True,

    entry_points="""\
    [console_scripts]
    bucky=bucky.main:main
    """
)



# -*- coding: utf-8 -
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import copy
import logging
import os
import socket
import struct
import sys
import threading


log = logging.getLogger(__name__)


class CollectDError(Exception):
    def __init__(self, mesg):
        self.mesg = mesg
    def __str__(self):
        return self.mesg


class ConfigError(CollectDError):
    pass


class ProtocolError(CollectDError):
    pass


class BindError(CollectDError):
    pass


class ServerErrror(CollectDError):
    pass


class CPUConverter(object):
    PRIORITY = 0
    def __call__(self, sample):
        return ["cpu", sample["plugin_instance"], sample["type_instance"]]


class InterfaceConverter(object):
    PRIORITY = 0
    def __call__(self, sample):
        return [
            "interface",
            sample["type_instance"],
            sample["type"],
            sample["value_name"]
        ]


class MemoryConverter(object):
    PRIORITY = 0
    def __call__(self, sample):
        return ["memory", sample["type_instance"]]


DEFAULT_CONVERTERS = {
    "cpu": CPUConverter(),
    "interface": InterfaceConverter(),
    "memory": MemoryConverter(),
}


class CollectDTypes(object):
    def __init__(self, types_db=None):
        self.types_db = types_db
        self.types = {}
        self.type_ranges = {}
        self._load_types()

    def get(self, name):
        t = self.types.get(name)
        if t is None:
            raise ProtocolError("Invalid type name: %s" % name)
        return t

    def _load_types(self):
        with open(self._fname()) as handle:
            for line in handle:
                if line.lstrip()[:1] == "#":
                    continue
                if not line.strip():
                    continue
                self._add_type_line(line)

    def _fname(self):
        if self.types_db is not None:
            return self.typesdb
        ret = "/usr/share/collectd/types.db"
        if os.path.exists(ret):
            return ret
        ret = "/usr/local/share/collectd/types.db"
        if os.path.exists(ret):
            return ret
        raise ConfigError("Unable to locate types.db")

    def _add_type_line(self, line):
        types = {
            "COUNTER": 0,
            "GAUGE": 1,
            "DERIVE": 2,
            "ABSOLUTE": 3
        }
        name, spec = line.split(None, 1)
        self.types[name] = []
        self.type_ranges[name] = {}
        vals = spec.split(", ")
        for val in vals:
            vname, vtype, minv, maxv = val.strip().split(":")
            vtype = types.get(vtype)
            if vtype is None:
                raise ValueError("Invalid value type: %s" % vtype)
            minv = None if minv == "U" else float(minv)
            maxv = None if maxv == "U" else float(maxv)
            self.types[name].append((vname, vtype))
            self.type_ranges[name][vname] = (minv, maxv)


class CollectDParser(object):
    def __init__(self, types_db=None):
        self.types = CollectDTypes(types_db=types_db)

    def parse(self, data):
        for sample in self.parse_samples(data):
            yield sample

    def parse_samples(self, data):
        types = {
            0x0000: self._parse_string("host"),
            0x0001: self._parse_time("time"),
            0x0008: self._parse_time_hires("time"),
            0x0002: self._parse_string("plugin"),
            0x0003: self._parse_string("plugin_instance"),
            0x0004: self._parse_string("type"),
            0x0005: self._parse_string("type_instance"),
            0x0006: None, # handle specially
            0x0007: self._parse_time("interval"),
            0x0009: self._parse_time_hires("interval")
        }
        sample = {}
        for (ptype, data) in self.parse_data(data):
            if ptype not in types:
                log.debug("Ignoring part type: 0x%02x" % ptype)
                continue
            if ptype != 0x0006:
                types[ptype](sample, data)
                continue
            for vname, vtype, val in self.parse_values(sample["type"], data):
                sample["value_name"] = vname
                sample["value_type"] = vtype
                sample["value"] = val
                yield copy.deepcopy(sample)

    def parse_data(self, data):
        types = set([
            0x0000, 0x0001, 0x0002, 0x0003, 0x0004,
            0x0005, 0x0006, 0x0007, 0x0008, 0x0009,
            0x0100, 0x0101, 0x0200, 0x0210
        ])
        while len(data) > 0:
            if len(data) < 4:
                raise ProtocolError("Truncated header.")
            (part_type, part_len) = struct.unpack("!HH", data[:4])
            data = data[4:]
            if part_type not in types:
                raise ProtocolError("Invalid part type: 0x%02x" % part_type)
            part_len -= 4 # includes four header bytes we just parsed
            if len(data) < part_len:
                raise ProtocolError("Truncated value.")
            part_data, data = data[:part_len], data[part_len:]
            yield (part_type, part_data)

    def parse_values(self, stype, data):
        types = {0: "!Q", 1: "<d", 2: "!q", 3: "!Q"}
        (nvals,) = struct.unpack("!H", data[:2])
        data = data[2:]
        if len(data) != 9 * nvals:
            raise ProtocolError("Invalid value structure length.")
        vtypes = self.types.get(stype)
        if nvals != len(vtypes):
            raise ProtocolError("Values different than types.db info.")
        for i in range(nvals):
            (vtype,) = struct.unpack("B", data[i])
            if vtype != vtypes[i][1]:
                raise ProtocolError("Type mismatch with types.db")
        data = data[nvals:]
        for i in range(nvals):
            vdata, data = data[:8], data[8:]
            (val,) = struct.unpack(types[vtypes[i][1]], vdata)
            yield vtypes[i][0], vtypes[i][1], val

    def _parse_string(self, name):
        def _parser(sample, data):
            if data[-1] != '\0':
                raise ProtocolError("Invalid string detected.")
            sample[name] = data[:-1]
        return _parser

    def _parse_time(self, name):
        def _parser(sample, data):
            if len(data) != 8:
                raise ProtocolError("Invalid time data length.")
            (val,) = struct.unpack("!Q", data)
            sample[name] = float(val)
        return _parser

    def _parse_time_hires(self, name):
        def _parser(sample, data):
            if len(data) != 8:
                raise ProtocolError("Invalid hires time data length.")
            (val,) = struct.unpack("!Q", data)
            sample[name] = val * (2 ** -30)
        return _parser


class CollectDConverter(object):
    def __init__(self, cfg):
        self.prefix = cfg.get("collectd_prefix")
        self.postfix = cfg.get("collectd_postfix")
        self.replace = cfg.get("collectd_replace", "_")
        self.strip_dupes = cfg.get("collectd_strip_duplicates", True)
        self.use_entry_points = cfg.get("collectd_use_entry_points", True)
        host_trim = cfg.get("collectd_host_trim", [])
        self.host_trim = []
        for s in host_trim:
            s = list(reversed(p.strip() for p in s.split(".")))
            self.strip.append(s)
        self.converters = dict(DEFAULT_CONVERTERS)
        self._load_converters(cfg)

    def convert(self, sample):
        stat = self.stat(sample)
        return stat, sample["value_type"], sample["value"], int(sample["time"])

    def stat(self, sample):
        parts = []
        if self.prefix:
            parts.append(self.prefix)
        parts.extend(self.hostname(sample.get("host", "")))
        handler = self.converters.get(sample["plugin"], self.default)
        parts.extend(handler(sample))
        if self.postfix:
            parts.append(self.postfix)
        if self.replace is not None:
            parts = [p.replace(".", self.replace) for p in parts]
        if self.strip_duplicates:
            parts = self.strip_duplicates(parts)
        return ".".join(parts)

    def hostname(self, host):
        parts = host.split(".")
        parts = list(reversed([p.strip() for p in parts]))
        for s in self.host_trim:
            same = True
            for i, p in enumerate(s):
                if p != parts[i]:
                    same = False
                    break
            if same:
                parts = parts[len(s):]
        return parts

    def default(self, sample):
        parts = []
        parts.append(sample["plugin"].strip())
        if sample.get("plugin_instance"):
            parts.append(sample["plugin_instance"].strip())
        stype = sample.get("type", "").strip()
        if stype and stype != "value":
            parts.append(stype)
        stypei = sample.get("type_instance", "").strip()
        if stypei:
            parts.append(stypei)
        vname = sample.get("value_name").strip()
        if vname and vname != "value":
            parts.append(vname)
        return parts

    def strip_duplicates(self, parts):
        ret = []
        for p in parts:
            if len(ret) == 0 or p != ret[-1]:
                ret.append(p)
        return ret

    def _load_converters(self, cfg):
        cfg_conv = cfg.get("collectd_converters", {})
        for conv in cfg_conv:
            self._add_converter(conv, cfg_conv[conv])
        if not cfg["collectd_use_entry_points"]:
            return
        import pkg_resources
        group = 'bucky.collectd.converters'
        for ep in pkg_resources.iter_entry_points(group):
            name, klass = ep.name, ep.load()
            self._add_converter(name, klass())

    def _add_converter(self, name, inst):
        if name not in self.converters:
            log.info("Converter: %s from %s" % (name, ep.module_name))
            self.converters[name] = klass()
            return
        kpriority = getattr(klass, "kpriority", 0)
        ipriority = getattr(self.converters[name], "PRIORITY", 0)
        if kpriority > ipriority:
            log.info("Replacing: %s" % name)
            log.info("Converter: %s from %s" % (name, ep.module_name))
            self.converters[name] = klass()
            return
        log.info("Ignoring: %s from %s" % (name, ep.module_name))


class CollectDServer(threading.Thread):
    def __init__(self, queue, cfg):
        super(CollectDServer, self).__init__()
        self.setDaemon(True)

        self.queue = queue
        self.parser = CollectDParser(cfg["collectd_types"])
        self.converter = CollectDConverter(cfg)
        self.sock = self.init_socket(cfg["collectd_ip"], cfg["collectd_port"])
        self.prev_samples = {}

    def init_socket(self, ip, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            sock.bind((ip, port))
            log.info("Opened collectd socket %s:%s" % (ip, port))
            return sock
        except Exception:
            log.error("Error opening collectd socket %s:%s." % (ip, port))
            sys.exit(1)

    def run(self):
        while True:
            data, addr = self.sock.recvfrom(65535)
            try:
                for sample in self.parser.parse(data):
                    self.last_sample = sample
                    name, vtype, val, time = self.converter.convert(sample)
                    if not name.strip():
                        continue
                    val = self.calculate(name, vtype, val, time)
                    if val is not None:
                        self.queue.put((name, val, time))
            except ProtocolError, e:
                log.error("Protocol error: %s" % e)
                log.info("Last sample: %s" % self.last_sample)

    def calculate(self, name, vtype, val, time):
        handlers = {
            0: self._calc_counter,  # counter
            1: lambda _name, v, _time: v,         # gauge
            2: self._calc_derive,  # derive
            3: self._calc_absolute  # absolute
        }
        if vtype not in handlers:
            log.error("Invalid value type %s for %s" % (vtype, name))
            log.info("Last sample: %s" % self.last_sample)
            return
        return handlers[vtype](name, val, time)

    def _calc_counter(self, name, val, time):
        # I need to figure out how to handle wrapping
        # Read: http://oss.oetiker.ch/rrdtool/tut/rrdtutorial.en.html
        # and then fix later
        if name not in self.prev_samples:
            self.prev_samples[name] = (val, time)
            return
        pval, ptime = self.prev_samples[name]
        self.prev_samples[name] = (val, time)
        if val < pval or time <= ptime:
            log.error("Invalid COUNTER update for: %s" % name)
            log.info("Last sample: %s" % self.last_sample)
            return
        return (val - pval) / (time - ptime)

    def _calc_derive(self, name, val, time):
        # Like counter, I need to figure out wrapping
        if name not in self.prev_samples:
            self.prev_samples[name] = (val, time)
            return
        pval, ptime = self.prev_samples[name]
        self.prev_samples[name] = (val, time)
        if time <= ptime:
            log.error("Invalid DERIVE update for: %s" % name)
            log.info("Last sample: %s" % self.last_sample)
            return
        return (val - pval) / (time - ptime)

    def _calc_absolute(self, name, val, time):
        if name not in self.prev_samples:
            self.prev_samples[name] = (val, time)
            return
        _pval, ptime = self.prev_samples[name]
        self.prev_samples[name] = (val, time)
        if time <= ptime:
            log.error("Invalid ABSOLUTE update for: %s" % name)
            log.info("Last sample: %s" % self.last_sample)
            return
        return val / (time - ptime)



# -*- coding: utf-8 -
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#
# Copyright 2011 Cloudant, Inc.

import logging
import optparse as op
import os
import Queue
import sys

import bucky
import bucky.carbon as carbon
import bucky.collectd as collectd
import bucky.statsd as statsd


log = logging.getLogger(__name__)
logfmt = "[%(levelname)s] %(module)s - %(message)s"
logging.basicConfig(format=logfmt, level=logging.DEBUG)


__usage__ = "%prog [OPTIONS] [CONFIG_FILE]"
__version__ = "bucky %s" % bucky.__version__

def options():
    return [
        op.make_option("--collectd-ip", dest="collectd_ip", metavar="IP",
            default="127.0.0.1",
            help="IP address to bind for the CollectD UDP socket [%default]"
        ),
        op.make_option("--collectd-port", dest="collectd_port", metavar="INT",
            type='int', default=25826,
            help="Port to bind for the CollectD UDP socket [%default]"
        ),
        op.make_option("--collectd-types", dest="collectd_types",
            metavar="FILE", default=None,
            help="Path to the collectd types.db file"
        ),
        op.make_option("--disable-collectd", dest="collectd_enabled",
            default=True, action="store_false",
            help="Disable the CollectD UDP server"
        ),
        op.make_option("--statsd-ip", dest="statsd_ip", metavar="IP",
            default="127.0.0.1",
            help="IP address to bind for the StatsD UDP socket [%default]"
        ),
        op.make_option("--statsd-port", dest="statsd_port", metavar="INT",
            type="int", default=8125,
            help="Port to bind for the StatsD UDP socket [%default]"
        ),
        op.make_option("--disable-statsd", dest="statsd_enabled",
            default=True, action="store_false",
            help="Disable the StatsD server"
        ),
        op.make_option("--graphite-ip", dest="graphite_ip", metavar="IP",
            default="127.0.0.1",
            help="IP address of the Graphite/Carbon server [%default]"
        ),
        op.make_option("--graphite-port", dest="graphite_port", metavar="INT",
            type="int", default=2003,
            help="Port of the Graphite/Carbon server [%default]"
        ),
        op.make_option("--full-trace", dest="full_trace",
            default=False, action="store_true",
            help="Display full error if config file fails to load"
        ),
    ]


def main():
    parser = op.OptionParser(usage=__usage__, version=__version__,
                                option_list=options())
    opts, args = parser.parse_args()

    if len(args) > 1:
        parser.error("Too many arguments.")
    if len(args) == 1 and not os.path.isfile(args[0]):
        parser.error("Invalid config file: %s" % opts.config)

    cfgfile = None
    if len(args) == 1:
        cfgfile = args[0]
    cfg = load_config(opts, cfgfile)

    sampleq = Queue.Queue()

    if opts.collectd_enabled:
        cdsrv = collectd.CollectDServer(sampleq, cfg)
        cdsrv.start()

    if opts.statsd_enabled:
        stsrv = statsd.StatsDServer(sampleq, cfg)
        stsrv.start()

    cli = carbon.CarbonClient(cfg)

    while True:
        try:
            stat, value, time = sampleq.get(True, 1)
            cli.send(stat, value, time)
        except Queue.Empty:
            pass
        if opts.collectd_enabled and not cdsrv.is_alive():
            log.error("collectd server died")
            break
        if opts.statsd_enabled and not stsrv.is_alive():
            log.error("statsd server died")
            break


def load_config(opts, cfgfile=None):
    cfg = {
        "__builtins__": __builtins__,
        "__name__": "__config__",
        "__file__": cfgfile,
        "__doc__": None,
        "__package__": None
    }
    param_names = """
        collectd_ip collectd_port collectd_types
        statsd_ip statsd_port
        graphite_ip graphite_port
    """.split()
    for name in param_names:
        cfg[name] = getattr(opts, name)
    cfg["collectd_converters"] = []
    cfg["collectd_use_entry_points"] = True
    cfg["statsd_flush_time"] = 10
    try:
        if cfgfile is not None:
            execfile(cfgfile, cfg, cfg)
    except Exception, e:
        log.error("Failed to read config file: %s" % cfgfile)
        log.error("Reason: %s" % e)
        if opts.full_trace:
            import traceback
            traceback.print_exc()
        sys.exit(1)
    return cfg


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass
    except Exception, e:
        print e
